@@ -8,9 +8,9 @@
 # Contact email: tomkralidis@gmail.com
 # =============================================================================
 
-""" ISO metadata parser """
+# flake8: noqa: E501
 
-import warnings
+""" ISO metadata parser """
 
 from owslib.etree import etree
 from owslib import util
@@ -33,6 +33,7 @@ class MD_Metadata(object):
     def __init__(self, md=None):
 
         if md is None:
+            self.md = None
             self.xml = None
             self.identifier = None
             self.parentidentifier = None
@@ -48,15 +49,14 @@ class MD_Metadata(object):
             self.stdver = None
             self.locales = []
             self.referencesystem = None
-            self.identification = None
             self.contentinfo = None
-            self.serviceidentification = None
-            self.identificationinfo = []
+            self.identification = []
             self.contentinfo = []
             self.distribution = None
             self.dataquality = None
             self.acquisition = None
         else:
+            self.md = md
             if hasattr(md, 'getroot'):  # standalone document
                 self.xml = etree.tostring(md.getroot())
             else:  # part of a larger document
@@ -114,38 +114,18 @@ class MD_Metadata(object):
             else:
                 self.referencesystem = None
 
-            # TODO: merge .identificationinfo into .identification
-            warnings.warn(
-                'the .identification and .serviceidentification properties will merge into '
-                '.identification being a list of properties.  This is currently implemented '
-                'in .identificationinfo.  '
-                'Please see https://github.com/geopython/OWSLib/issues/38 for more information',
-                FutureWarning)
+            self.identification = []
 
-            val = md.find(util.nspath_eval('gmd:identificationInfo/gmd:MD_DataIdentification', namespaces))
-            val2 = md.find(util.nspath_eval('gmd:identificationInfo/srv:SV_ServiceIdentification', namespaces))
-
-            if val is not None:
-                self.identification = MD_DataIdentification(val, 'dataset')
-                self.serviceidentification = None
-            elif val2 is not None:
-                self.identification = MD_DataIdentification(val2, 'service')
-                self.serviceidentification = SV_ServiceIdentification(val2)
-            else:
-                self.identification = None
-                self.serviceidentification = None
-
-            self.identificationinfo = []
             for idinfo in md.findall(util.nspath_eval('gmd:identificationInfo', namespaces)):
                 if len(idinfo) > 0:
                     val = list(idinfo)[0]
                     tagval = util.xmltag_split(val.tag)
                     if tagval == 'MD_DataIdentification':
-                        self.identificationinfo.append(MD_DataIdentification(val, 'dataset'))
+                        self.identification.append(MD_DataIdentification(val, 'dataset'))
                     elif tagval == 'MD_ServiceIdentification':
-                        self.identificationinfo.append(MD_DataIdentification(val, 'service'))
+                        self.identification.append(MD_DataIdentification(val, 'service'))
                     elif tagval == 'SV_ServiceIdentification':
-                        self.identificationinfo.append(SV_ServiceIdentification(val))
+                        self.identification.append(SV_ServiceIdentification(val))
 
             self.contentinfo = []
             for contentinfo in md.findall(
@@ -174,6 +154,22 @@ class MD_Metadata(object):
             else:
                 self.acquisition = None
 
+    def get_all_contacts(self):
+        """get all contacts in document"""
+
+        contacts = []
+
+        for ii in self.identification:
+            for iic in ii.contact:
+                contacts.append(iic)
+
+            for ct in ['creator', 'publisher', 'contributor']:
+                iict = getattr(ii, ct)
+                if iict:
+                    contacts.append(iict)
+
+        return list(filter(None, contacts))
+
     def get_default_locale(self):
         """ get default gmd:PT_Locale based on gmd:language """
 
@@ -197,7 +193,7 @@ class PT_Locale(object):
                 util.nspath_eval('gmd:languageCode/gmd:LanguageCode', namespaces)).attrib.get('codeListValue')
             self.charset = md.find(
                 util.nspath_eval('gmd:characterEncoding/gmd:MD_CharacterSetCode', namespaces)).attrib.get(
-                    'codeListValue')
+                'codeListValue')
 
 
 class CI_Date(object):
@@ -284,7 +280,7 @@ class CI_ResponsibleParty(object):
             self.country = util.testXMLValue(val)
 
             val = md.find(util.nspath_eval(
-                'gmd:contactInfo/gmd:CI_Contact/gmd:address/gmd:CI_Address/gmd:electronicMailAddress/gco:CharacterString',  # noqa
+                'gmd:contactInfo/gmd:CI_Contact/gmd:address/gmd:CI_Address/gmd:electronicMailAddress/gco:CharacterString',
                 namespaces))
             self.email = util.testXMLValue(val)
 
@@ -298,6 +294,19 @@ class CI_ResponsibleParty(object):
             self.role = _testCodeListValue(md.find(util.nspath_eval('gmd:role/gmd:CI_RoleCode', namespaces)))
 
 
+class Keyword(object):
+    """
+    Class for complex keywords, with labels and URLs
+    """
+    def __init__(self, kw=None):
+        if kw is None:
+            self.name = None
+            self.url = None
+        else:
+            self.name = util.testXMLValue(kw)
+            self.url = kw.attrib.get(util.nspath_eval('xlink:href', namespaces))
+
+
 class MD_Keywords(object):
     """
     Class for the metadata MD_Keywords element
@@ -307,13 +316,14 @@ class MD_Keywords(object):
             self.keywords = []
             self.type = None
             self.thesaurus = None
-            self.kwdtype_codeList = 'http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/codelist/gmxCodelists.xml#MD_KeywordTypeCode'  # noqa
+            self.kwdtype_codeList = 'http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/codelist/gmxCodelists.xml#MD_KeywordTypeCode'
         else:
             self.keywords = []
             val = md.findall(util.nspath_eval('gmd:keyword/gco:CharacterString', namespaces))
+            if len(val) == 0:
+                val = md.findall(util.nspath_eval('gmd:keyword/gmx:Anchor', namespaces))
             for word in val:
-                self.keywords.append(util.testXMLValue(word))
-
+                self.keywords.append(Keyword(word))
             self.type = None
             val = md.find(util.nspath_eval('gmd:type/gmd:MD_KeywordTypeCode', namespaces))
             self.type = util.testXMLAttribute(val, 'codeListValue')
@@ -323,8 +333,8 @@ class MD_Keywords(object):
             if val is not None:
                 self.thesaurus = {}
 
-                thesaurus = val.find(util.nspath_eval('gmd:title/gco:CharacterString', namespaces))
-                self.thesaurus['title'] = util.testXMLValue(thesaurus)
+                title = val.find(util.nspath_eval('gmd:title/gco:CharacterString', namespaces))
+                self.thesaurus['title'] = util.testXMLValue(title)
                 self.thesaurus['url'] = None
 
                 if self.thesaurus['title'] is None:  # try gmx:Anchor
@@ -333,12 +343,12 @@ class MD_Keywords(object):
                         self.thesaurus['title'] = util.testXMLValue(t)
                         self.thesaurus['url'] = t.attrib.get(util.nspath_eval('xlink:href', namespaces))
 
-                thesaurus = val.find(util.nspath_eval('gmd:date/gmd:CI_Date/gmd:date/gco:Date', namespaces))
-                self.thesaurus['date'] = util.testXMLValue(thesaurus)
+                date_ = val.find(util.nspath_eval('gmd:date/gmd:CI_Date/gmd:date/gco:Date', namespaces))
+                self.thesaurus['date'] = util.testXMLValue(date_)
 
-                thesaurus = val.find(
+                datetype = val.find(
                     util.nspath_eval('gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode', namespaces))
-                self.thesaurus['datetype'] = util.testXMLAttribute(thesaurus, 'codeListValue')
+                self.thesaurus['datetype'] = util.testXMLAttribute(datetype, 'codeListValue')
 
 
 class MD_DataIdentification(object):
@@ -373,9 +383,9 @@ class MD_DataIdentification(object):
             self.abstract_url = None
             self.purpose = None
             self.status = None
+            self.graphicoverview = []
             self.contact = []
             self.keywords = []
-            self.keywords2 = []
             self.topiccategory = []
             self.supplementalinformation = None
             self.extent = None
@@ -486,7 +496,7 @@ class MD_DataIdentification(object):
 
             self.denominators = []
             for i in md.findall(util.nspath_eval(
-                    'gmd:spatialResolution/gmd:MD_Resolution/gmd:equivalentScale/gmd:MD_RepresentativeFraction/gmd:denominator/gco:Integer',  # noqa
+                    'gmd:spatialResolution/gmd:MD_Resolution/gmd:equivalentScale/gmd:MD_RepresentativeFraction/gmd:denominator/gco:Integer',
                     namespaces)):
                 val = util.testXMLValue(i)
                 if val is not None:
@@ -528,7 +538,7 @@ class MD_DataIdentification(object):
                     elif clv == 'author':
                         self.contributor.append(rp)
 
-            val = md.find(util.nspath_eval('gmd:edition/gco:CharacterString', namespaces))
+            val = md.find(util.nspath_eval('gmd:citation/gmd:CI_Citation/gmd:edition/gco:CharacterString', namespaces))
             self.edition = util.testXMLValue(val)
 
             val = md.find(util.nspath_eval('gmd:abstract/gco:CharacterString', namespaces))
@@ -546,6 +556,14 @@ class MD_DataIdentification(object):
 
             self.status = _testCodeListValue(md.find(util.nspath_eval('gmd:status/gmd:MD_ProgressCode', namespaces)))
 
+            self.graphicoverview = []
+            for val in md.findall(util.nspath_eval(
+                    'gmd:graphicOverview/gmd:MD_BrowseGraphic/gmd:fileName/gco:CharacterString', namespaces)):
+                if val is not None:
+                    val2 = util.testXMLValue(val)
+                    if val2 is not None:
+                        self.graphicoverview.append(val2)
+
             self.contact = []
             for i in md.findall(util.nspath_eval('gmd:pointOfContact/gmd:CI_ResponsibleParty', namespaces)):
                 o = CI_ResponsibleParty(i)
@@ -558,50 +576,9 @@ class MD_DataIdentification(object):
                 if val:
                     self.spatialrepresentationtype.append(val)
 
-            warnings.warn(
-                'The .keywords and .keywords2 properties will merge into the '
-                '.keywords property in the future, with .keywords becoming a list '
-                'of MD_Keywords instances. This is currently implemented in .keywords2. '
-                'Please see https://github.com/geopython/OWSLib/issues/301 for more information',
-                FutureWarning)
-
             self.keywords = []
-
-            for i in md.findall(util.nspath_eval('gmd:descriptiveKeywords', namespaces)):
-                mdkw = {}
-                mdkw['type'] = _testCodeListValue(i.find(util.nspath_eval(
-                    'gmd:MD_Keywords/gmd:type/gmd:MD_KeywordTypeCode', namespaces)))
-
-                mdkw['thesaurus'] = {}
-
-                val = i.find(util.nspath_eval(
-                    'gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:title/gco:CharacterString', namespaces))
-                mdkw['thesaurus']['title'] = util.testXMLValue(val)
-
-                val = i.find(util.nspath_eval(
-                    'gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:date/gco:Date',
-                    namespaces))
-                mdkw['thesaurus']['date'] = util.testXMLValue(val)
-
-                val = i.find(util.nspath_eval(
-                    'gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode',  # noqa
-                    namespaces))
-                mdkw['thesaurus']['datetype'] = util.testXMLAttribute(val, 'codeListValue')
-
-                mdkw['keywords'] = []
-
-                for k in i.findall(util.nspath_eval('gmd:MD_Keywords/gmd:keyword', namespaces)):
-                    val = k.find(util.nspath_eval('gco:CharacterString', namespaces))
-                    if val is not None:
-                        val2 = util.testXMLValue(val)
-                        if val2 is not None:
-                            mdkw['keywords'].append(val2)
-
-                self.keywords.append(mdkw)
-
-            self.keywords2 = []
             for mdkw in md.findall(util.nspath_eval('gmd:descriptiveKeywords/gmd:MD_Keywords', namespaces)):
-                self.keywords2.append(MD_Keywords(mdkw))
+                self.keywords.append(MD_Keywords(mdkw))
 
             self.topiccategory = []
             for i in md.findall(util.nspath_eval('gmd:topicCategory/gmd:MD_TopicCategoryCode', namespaces)):
@@ -632,21 +609,21 @@ class MD_DataIdentification(object):
 
                 if val2 is None:
                     val2 = extent.find(util.nspath_eval(
-                        'gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:beginPosition',  # noqa
+                        'gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:beginPosition',
                         namespaces))
                     if val2 is None:
                         val2 = extent.find(util.nspath_eval(
-                            'gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml32:TimePeriod/gml32:beginPosition',  # noqa
+                            'gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml32:TimePeriod/gml32:beginPosition',
                             namespaces))
                     self.temporalextent_start = util.testXMLValue(val2)
 
                 if val3 is None:
                     val3 = extent.find(util.nspath_eval(
-                        'gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:endPosition',  # noqa
+                        'gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml:TimePeriod/gml:endPosition',
                         namespaces))
                     if val3 is None:
                         val3 = extent.find(util.nspath_eval(
-                            'gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml32:TimePeriod/gml32:endPosition',  # noqa
+                            'gmd:EX_Extent/gmd:temporalElement/gmd:EX_TemporalExtent/gmd:extent/gml32:TimePeriod/gml32:endPosition',
                             namespaces))
                     self.temporalextent_end = util.testXMLValue(val3)
 
@@ -667,7 +644,7 @@ class MD_Distributor(object):
             self.online = []
 
             for ol in md.findall(util.nspath_eval(
-                'gmd:MD_Distributor/gmd:distributorTransferOptions/gmd:MD_DigitalTransferOptions/gmd:onLine/gmd:CI_OnlineResource',  # noqa
+                    'gmd:MD_Distributor/gmd:distributorTransferOptions/gmd:MD_DigitalTransferOptions/gmd:onLine/gmd:CI_OnlineResource',
                     namespaces)):
                 self.online.append(CI_OnlineResource(ol))
 
@@ -717,7 +694,7 @@ class DQ_DataQuality(object):
         else:
             self.conformancetitle = []
             for i in md.findall(util.nspath_eval(
-                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:title/gco:CharacterString',  # noqa
+                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:title/gco:CharacterString',
                     namespaces)):
                 val = util.testXMLValue(i)
                 if val is not None:
@@ -725,7 +702,7 @@ class DQ_DataQuality(object):
 
             self.conformancedate = []
             for i in md.findall(util.nspath_eval(
-                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:date/gco:Date',  # noqa
+                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:date/gco:Date',
                     namespaces)):
                 val = util.testXMLValue(i)
                 if val is not None:
@@ -733,7 +710,7 @@ class DQ_DataQuality(object):
 
             self.conformancedatetype = []
             for i in md.findall(util.nspath_eval(
-                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode',  # noqa
+                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode',
                     namespaces)):
                 val = _testCodeListValue(i)
                 if val is not None:
@@ -757,48 +734,33 @@ class DQ_DataQuality(object):
                 self.lineage_url = val.attrib.get(util.nspath_eval('xlink:href', namespaces))
 
             val = md.find(util.nspath_eval(
-                'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:title/gco:CharacterString',  # noqa
+                'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:title/gco:CharacterString',
                 namespaces))
             self.specificationtitle = util.testXMLValue(val)
 
             self.specificationdate = []
             for i in md.findall(util.nspath_eval(
-                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:date/gmd:CI_Date',  # noqa
+                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:date/gmd:CI_Date',
                     namespaces)):
                 val = util.testXMLValue(i)
                 if val is not None:
                     self.specificationdate.append(val)
 
 
-class SV_ServiceIdentification(object):
+class SV_ServiceIdentification(MD_DataIdentification):
     """ process SV_ServiceIdentification """
     def __init__(self, md=None):
+
+        super().__init__(md, 'service')
+
         if md is None:
-            self.title = None
-            self.abstract = None
-            self.contact = None
-            self.identtype = 'service'
             self.type = None
             self.version = None
             self.fees = None
-            self.bbox = None
             self.couplingtype = None
             self.operations = []
             self.operateson = []
         else:
-            val = md.find(util.nspath_eval('gmd:citation/gmd:CI_Citation/gmd:title/gco:CharacterString', namespaces))
-            self.title = util.testXMLValue(val)
-
-            val = md.find(util.nspath_eval('gmd:abstract/gco:CharacterString', namespaces))
-            self.abstract = util.testXMLValue(val)
-
-            self.contact = None
-            val = md.find(util.nspath_eval(
-                'gmd:citation/gmd:CI_Citation/gmd:citedResponsibleParty/gmd:CI_ResponsibleParty', namespaces))
-            if val is not None:
-                self.contact = CI_ResponsibleParty(val)
-
-            self.identtype = 'service'
             val = md.find(util.nspath_eval('srv:serviceType/gco:LocalName', namespaces))
             self.type = util.testXMLValue(val)
 
@@ -809,13 +771,6 @@ class SV_ServiceIdentification(object):
                 'srv:accessProperties/gmd:MD_StandardOrderProcess/gmd:fees/gco:CharacterString', namespaces))
             self.fees = util.testXMLValue(val)
 
-            val = md.find(util.nspath_eval('srv:extent/gmd:EX_Extent', namespaces))
-
-            if val is not None:
-                self.bbox = EX_Extent(val)
-            else:
-                self.bbox = None
-
             self.couplingtype = _testCodeListValue(md.find(util.nspath_eval(
                 'gmd:couplingType/gmd:SV_CouplingType', namespaces)))
 
@@ -955,7 +910,7 @@ class EX_Extent(object):
                     self.boundingPolygon = EX_GeographicBoundingPolygon(polygonElement)
 
                 val = md.find(util.nspath_eval(
-                    'gmd:EX_GeographicDescription/gmd:geographicIdentifier/gmd:MD_Identifier/gmd:code/gco:CharacterString',  # noqa
+                    'gmd:EX_GeographicDescription/gmd:geographicIdentifier/gmd:MD_Identifier/gmd:code/gco:CharacterString',
                     namespaces))
                 self.description_code = util.testXMLValue(val)
 
