@@ -12,8 +12,6 @@ Conceptual model base classes: http://www.opengeospatial.org/standards/owc
 OGC OWS Context Conceptual Model 1.0 (12-080r2)
 """
 
-from __future__ import (absolute_import, division, print_function)
-
 from owslib.owscontext.atom import decode_atomxml, encode_atomxml
 from owslib.owscontext.common import GENERIC_OWCSPEC_URL
 # from owslib.util import log
@@ -143,8 +141,7 @@ class OwcContext(object):
                         [obj.to_dict() for obj in self.spec_reference],
                     "via":
                         [] if len(self.context_metadata) <= 0 else
-                        [obj.to_dict() for obj in self.context_metadata]
-                    },
+                        [obj.to_dict() for obj in self.context_metadata]},
             },
             "features":
                 [] if len(self.resources) <= 0 else
@@ -341,8 +338,7 @@ class OwcResource(object):
                     "via":
                         [] if len(self.resource_metadata) <= 0 else
                         [obj.to_dict() for obj in
-                         self.resource_metadata]
-                    },
+                         self.resource_metadata]},
                 "offerings":
                     [] if len(self.offerings) <= 0 else
                     [obj.to_dict() for obj in self.offerings],
@@ -378,8 +374,7 @@ class OwcResource(object):
             resource_metadata=[OwcLink.from_dict(do) for do in
                                extract_p('properties.links.via', d, [])],
             content_description=[OwcLink.from_dict(do)
-                                 for do in extract_p(
-                    'properties.links.alternates', d, [])],
+                                 for do in extract_p('properties.links.alternates', d, [])],
             preview=[OwcLink.from_dict(do) for do in
                      extract_p('properties.links.previews', d, [])],
             content_by_ref=[OwcLink.from_dict(do) for do in
