@@ -180,7 +180,7 @@ class WebMapTileService(object):
         # Authentication handled by Reader
         reader = WMTSCapabilitiesReader(
             self.version, url=self.url, headers=self.headers, auth=self.auth)
-        if xml:  # read from stored xml
+        if xml is not None:  # read from stored xml
             self._capabilities = reader.readString(xml)
         else:  # read from server
             self._capabilities = reader.read(self.url, self.vendor_kwargs)
